/*******************************************************************************
                     Computación Paralela 2012 - FaMAF
                      Universidad Nacional de Córdoba

Ejercicio 1 - Grids 1D
======================
El siguiente código pinta una imagen y la muestra en pantalla.
Esto se hace corriendo el kernel block_gradient_1d, que donde cada thread pinta
una celda de un color único al bloque de threads al que pertenece.
El kernel asume que los threads están dispuestos en un solo grid unidimensional,
donde los bloques también son unidimensionales. Luego se copia la imagen al host
y se la muestra.

Complete el código definiendo el tamaño del grid para el tamaño de bloque
provisto y escribiendo la llamada al kernel. No se olvide de ningún punto!

********************************************************************************/

#include <cuda.h>   // API de CUDA
#include <cutil.h>  // macros para verificar llamadas
#include <stdio.h>
#include <stdlib.h>

#include "colorramp.h"
#include "sdlstuff.h"

#define WIDTH 181
#define HEIGHT 101
#define SIZE (WIDTH * HEIGHT)


__global__ void block_gradient_1d(rgba32 * image, size_t n) {

    // indice del thread en el vector
    size_t idx = blockIdx.x * blockDim.x + threadIdx.x;

    if (idx < n) {
        // color del bloque [0..1]
        float color = (float) blockIdx.x / (gridDim.x - 1);
        // pasar a rgb
        rgba32 rgba = color_1d(color);
        // escribir a memoria
        image[idx] = rgba;
    }
}


int main(int argc, char ** argv) {

    // inicializar sdl
    sdls_init(WIDTH, HEIGHT);

    // pedir memoria en la placa para la imagen e inicializar a 0
    rgba32 * device_image;
    CUDA_SAFE_CALL(cudaMalloc(&device_image, SIZE * sizeof(rgba32)));
    CUDA_SAFE_CALL(cudaMemset(device_image, 0, SIZE * sizeof(rgba32)));

    // correr kernel de gradiente
    dim3 block(1024);   // bloques de 1024 threads
    // FALTA: configurar grid!
    // FALTA: llamar kernel!

    CUT_CHECK_ERROR("Fallo al lanzar el kernel:");

    // esperar a que el kernel termine
    CUDA_SAFE_CALL(cudaDeviceSynchronize());

    // pedir memoria en el host para la imagen y copiar desde la placa
    rgba32 * host_image;
    host_image = (rgba32 *) malloc(SIZE * sizeof(rgba32));
    CUDA_SAFE_CALL(cudaMemcpy(host_image, device_image, SIZE * sizeof(rgba32), cudaMemcpyDefault));

    // dibujar en pantalla
    sdls_drawrectangle(0, 0, WIDTH, HEIGHT, host_image);

    // esperar input
    printf("<ENTER> para salir\n");
    getchar();

    // limpieza
    free(host_image);
    CUDA_SAFE_CALL(cudaFree(device_image));

    return 0;
}
